/*
 * Copyright (c) 2005-2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */

package org.acceleo.gen.phantom;

import fr.obeo.acceleo.gen.template.eval.SyncElement;

import java.io.Externalizable;
import java.io.File;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;


/**
 * This phantom structure is used with the synchronization tools of acceleo.
 * This structure implements the interface Serializable, so we can save it in a
 * stream.
 * 
 * @author www.obeo.fr
 * 
 */
public abstract class AbstractSyncPhantom implements Externalizable {

	/**
	 * The identifier of the internal extension point specifying the
	 * implementation to use with acceleo synchronization tools.
	 */
	public static final String SYNC_PHANTOM_EXTENSION_ID = "fr.obeo.acceleo.gen.syncphantom"; //$NON-NLS-1$

	/**
	 * The folder to put the generated phantoms.
	 */
	public static final String PHANTOM_GENERATED_FOLDER = PhantomResources.getRootName() + "/generated"; //$NON-NLS-1$

	/**
	 * The extension of the phantoms.
	 */
	public static final String PHANTOM_SYNC_FILE_EXTENSION = "sync"; //$NON-NLS-1$

	/**
	 * The project relative path of this file.
	 */
	protected String projectRelativePath;

	/**
	 * The model path.
	 */
	protected String model;

	/**
	 * The template path.
	 */
	protected String template;

	/**
	 * The target path.
	 */
	protected String target;

	/**
	 * The chain path.
	 */
	protected String chain;

	/**
	 * The generation time.
	 */
	protected long time;

	/**
	 * Returns the chain.
	 * 
	 * @return the chain.
	 */
	public String getChain() {
		return chain;
	}

	public void setChain(String chain) {
		this.chain = chain;
	}

	/**
	 * Puts the given structure in the phantom.
	 * 
	 * @param syncElement
	 *            is the structure to put in the phantom
	 */
	public abstract void setEval(SyncElement syncElement);

	public String getModel() {
		return model;
	}

	public void setModel(String model) {
		this.model = model;
	}

	public String getProjectRelativePath() {
		return projectRelativePath;
	}

	public void setProjectRelativePath(String projectRelativePath) {
		this.projectRelativePath = projectRelativePath;
	}

	public String getTemplate() {
		return template;
		// return Resources.decodeAcceleoAbsolutePath(template);
	}

	public void setTemplate(String template) {
		this.template = template;
		// this.template = Resources.encodeAcceleoAbsolutePath(template);
	}

	public String getTarget() {
		return target;
	}

	public void setTarget(String target) {
		this.target = target;
	}

	public long getTime() {
		return time;
	}

	public void setTime(long time) {
		this.time = time;
	}

	/**
	 * It uses the extension points to create phantom.
	 * 
	 * @param file
	 *            is the generated file
	 * @param model
	 *            is the model file
	 * @param template
	 *            is the template file
	 * @param target
	 *            is the target container
	 * @param chain
	 *            is the chain file
	 * @param eval
	 *            is the node to synchronize
	 * @throws IOException
	 *             If an I/O error has occurred
	 */
	// TODO progress monitor
	public static void createPhantomExtensionPoint(File file, File model, File template, File target, File chain, SyncElement eval/*
																																	 * ,
																																	 * IProgressMonitor
																																	 * monitor
																																	 */) throws IOException {
		createPhantomExtensionPoint(file, model, template, target, chain, eval, 0/*
																					 * ,
																					 * monitor
																					 */);
	}

	/**
	 * It uses the extension points to create phantom.
	 * 
	 * @param file
	 *            is the generated file
	 * @param model
	 *            is the model file
	 * @param template
	 *            is the template file
	 * @param target
	 *            is the target container
	 * @param chain
	 *            is the chain file
	 * @param eval
	 *            is the node to synchronize
	 * @param time
	 *            is the time to generate the file
	 * @throws IOException
	 *             If an I/O error has occurred
	 */
	// TODO progress monitor
	public static void createPhantomExtensionPoint(File file, File model, File template, File target, File chain, SyncElement eval, long time/*
																																				 * ,
																																				 * IProgressMonitor
																																				 * monitor
																																				 */) throws IOException {
		// TODO should give a folder to use instead of the project (the output
		// root directory)
		// IPath path = new
		// Path(PHANTOM_GENERATED_FOLDER).append(file.getProjectRelativePath()).addFileExtension(PHANTOM_SYNC_FILE_EXTENSION);
		// SyncPhantom phantom = null;
		// for (IPhantomExtension member : Extension.getPhantomExtension()) {
		// String projectNature = member.getProjectNature();
		// if (projectNature != null &&
		// file.getProject().hasNature(projectNature)) {
		// SyncElement syncClass = member.getSyncElement();
		// if (syncClass != null && syncClass.equals(eval.getClass().getName()))
		// {
		// SyncPhantom phantomClass = member.getSyncPhantom();
		// if (phantomClass != null) {
		// phantom.setProjectRelativePath(path.toString());
		// phantom.setModel((model != null) ? model.getPath() : "");
		// //$NON-NLS-1$
		// phantom.setTemplate((template != null) ? template.getAbsolutePath() :
		// ""); //$NON-NLS-1$
		// phantom.setTarget((target != null) ? target.getPath() : "");
		// //$NON-NLS-1$
		// phantom.setChain((chain != null) ? chain.getPath() : "");
		// //$NON-NLS-1$
		// phantom.setTime(time);
		// phantom.setEval(eval);
		// } else {
		// // TODO message - no phantom class found
		// }
		// }
		// }
		// }
		// if (phantom != null) {
		// // TODO file writer for the all project
		// //Resources.createFile(file.getProject(), path, phantom, monitor);
		// }
	}

	/**
	 * {@inheritDoc}
	 */
	public void readExternal(ObjectInput arg) throws IOException, ClassNotFoundException {
		projectRelativePath = arg.readUTF();
		model = arg.readUTF();
		template = arg.readUTF();
		target = arg.readUTF();
		chain = arg.readUTF();
		time = arg.readLong();
	}

	/**
	 * {@inheritDoc}
	 */
	public void writeExternal(ObjectOutput arg) throws IOException {
		arg.writeUTF(projectRelativePath);
		arg.writeUTF(model);
		arg.writeUTF(template);
		arg.writeUTF(target);
		arg.writeUTF(chain);
		arg.writeLong(time);
	}

}
