/*
 * Copyright (c) 2005-2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */

package org.acceleo.gen.phantom;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.util.ArrayList;
import java.util.List;

import org.acceleo.tools.resources.FileContentMap;

/**
 * The .launcher files provider.
 * 
 * @author www.obeo.fr
 * 
 */
public final class ChainPhantomProvider {

	/**
	 * The phantom path relative to the project.
	 */
	private static final String PHANTOM_CHAIN_FILE = PhantomResources.getRootName() + "/.launcher"; //$NON-NLS-1$

	/**
	 * The sole instance.
	 */
	private static ChainPhantomProvider instance;

	/**
	 * Gets the phantom of the container.
	 */
	private FileContentMap project2Phantom = new FileContentMap();

	/**
	 * Projects list.
	 */
	private List<File> projects = new ArrayList<File>();

	// private File lastChain = null;

	/**
	 * Constructor.
	 */
	private ChainPhantomProvider() {
	}

	/**
	 * Gets the sole instance.
	 * 
	 * @return the sole instance
	 */
	public static ChainPhantomProvider getDefault() {
		if (instance == null) {
			instance = new ChainPhantomProvider();
		}
		return instance;
	}

	/**
	 * Gets the phantom of the given project.
	 * 
	 * @param project
	 *            is the project
	 * @return the phantom or an empty phantom if it doesn't exist
	 * @throws IOException
	 *             If an I/O error has occurred
	 */
	private ChainPhantom getPhantom(File project) throws IOException {
		String path = project + File.separator + PHANTOM_CHAIN_FILE;

		File phantomChainFile = new File(path);
		ChainPhantom phantom = (ChainPhantom) project2Phantom.get(phantomChainFile);
		if (phantom == null) {
			phantom = readPhantom(phantomChainFile);
			if (phantom == null) {
				phantom = new ChainPhantom();
			}
			project2Phantom.put(phantomChainFile, phantom);
		}
		return phantom;
	}

	/**
	 * Gets the phantom in the given file.
	 * 
	 * @param file
	 *            is the phantom file
	 * @return the phantom or null if it doesn't exist
	 * @throws IOException
	 *             If an I/O error has occurred
	 */
	private ChainPhantom readPhantom(File file) throws IOException {
		ChainPhantom filePhantom;
		if (file != null && file.exists()) {
			FileInputStream fis = null;
			ObjectInputStream in = null;
			try {
				fis = new FileInputStream(file.getAbsolutePath());
				in = new ObjectInputStream(fis);
				ChainPhantom result = (ChainPhantom) in.readObject();
				in.close();
				filePhantom = result;
			} catch (ClassNotFoundException e) {
				// TODO exception management
				filePhantom = null;
				// throw new CoreException(new Status(IStatus.ERROR,
				// AcceleoEcoreGenPlugin.getDefault().getID(), -1,
				// AcceleoGenMessages.getString("ChainPhantomProvider.InvalidFile"),
				// e)); //$NON-NLS-1$
			}
		} else {
			filePhantom = null;
		}
		return null;
	}

	/**
	 * Saves the phantom in the given project.
	 * 
	 * @param project
	 *            is the project
	 * @param phantom
	 *            is the phantom to save
	 * @throws IOException
	 *             If an I/O error has occurred
	 */
	// TODO progress monitor
	private void savePhantom(File project, ChainPhantom phantom/*
																 * ,
																 * IProgressMonitor
																 * monitor
																 */) throws IOException {
		// boolean projectIsValid = false;
		// TODO can it be done stand alone ?
		// for (IPhantomExtension phantomExt : Extension.getPhantomExtension())
		// {
		// if (phantomExt.getProjectNature() != null &&
		// project.hasNature(phantomExt.getProjectNature())) {
		// projectIsValid = true;
		// }
		// }
		// if (projectIsValid) {
		// Resources.createFile(project, new Path(PHANTOM_CHAIN_FILE), phantom,
		// monitor);
		// }
	}

	/**
	 * Adds files generated by the given chain.
	 * 
	 * @param chain
	 *            is the chain
	 * @param generated
	 *            are the generated files
	 * @throws IOException
	 *             If an I/O error has occurred
	 */
	// TODO progress monitor
	public void add(File chain, File[] generated/* , IProgressMonitor monitor */) throws IOException {
		// TODO should give a folder to use instead of the project (the output
		// root directory)
		// File project = null;
		// ChainPhantom phantom = null;
		// for (int i = 0; i < generated.length; i++) {
		// if (project == null) {
		// project = generated[i].getProject();
		// phantom = getPhantom(project);
		// } else if (!project.equals(generated[i].getProject())) {
		// savePhantom(project, phantom/*, monitor*/);
		// project = generated[i].getProject();
		// phantom = getPhantom(project);
		// }
		// phantom.add(chain, generated[i]);
		// }
		// if (project != null) {
		// savePhantom(project, phantom/*, monitor*/);
		// }
	}

	/**
	 * Clears the context for the next update operation of the given chain.
	 * 
	 * @param chain
	 *            is the chain
	 */
	public void clearForUpdate(File chain) {
		projects.clear();
		// lastChain = chain;
	}

	/**
	 * Operation that replaces the generated files in the phantom.
	 * 
	 * @param chain
	 *            is the chain
	 * @param generated
	 *            are the generated files
	 * @throws IOException
	 *             If an I/O error has occurred
	 */
	// TODO progress monitor
	public void update(File chain, File[] generated/*
													 * , IProgressMonitor
													 * monitor
													 */) throws IOException {
		// TODO should give a folder to use instead of the project (the output
		// root directory)
		// if (chain != null && chain.equals(lastChain)) {
		// for (int i = 0; i < generated.length; i++) {
		// File project = generated[i].getProject();
		// if (!projects.contains(project)) {
		// projects.add(project);
		// clear(project, chain/*, monitor*/);
		// }
		// }
		// add(chain, generated/*, monitor*/);
		// }
	}

	/**
	 * Clears all information about the chain.
	 * 
	 * @param project
	 *            Project
	 * @param chain
	 *            Chain
	 * @throws IOException
	 *             If an I/O error has occurred
	 */
	private void clear(File project, File chain/* , IProgressMonitor monitor */) throws IOException {
		ChainPhantom phantom = getPhantom(project);
		phantom.clear(chain);
		savePhantom(project, phantom/* , monitor */);
	}

	/**
	 * Indicates if the given file has been generated.
	 * 
	 * @param generated
	 *            is the file to test
	 * @return true if the given file has been generated
	 * @throws IOException
	 *             If an I/O error has occurred
	 */
	public boolean isGenerated(File generated) throws IOException {
		// TODO should give a folder to use instead of the project (the output
		// root directory)
		// ChainPhantom phantom = getPhantom(generated.getProject());
		// return phantom.isGenerated(generated);
		return false;
	}

}
